/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : SPI_Slave.c

 Hardware      : ADuC706x

 Description   : This SPI Slave example shows how to initialise the SPI peripheral in Slave mode.

		
*************************************************************************************************/
#include<aduc7060.h>

// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000

unsigned char szStoreData[] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55};	// Array to send to Slave
unsigned char ucStoreCount = 0;  // Array index variable for szStoreData[]
unsigned char ucWaitingForXIRQ0 = 1; // Flag to begin write from Slave - controlled by XIRQ0
int main(void)
{
    POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;
	// Configure P0.0, P0.1,P0.2 and P0.3 for SPI mode
	GP0CON0 = BIT0 + BIT4 + BIT8 + BIT12; // Select SPI/I2C alternative function for P0[0...3]
	GP0KEY1 = 0x7;		   //Write to GP0KEY1
	GP0CON1	= 0x00;	   // Select SPI functionality for P0.0 to  P0.3
	GP0KEY2 = 0x13;		   //Write to GP0KEY2

	// Enable SPI Slave mode and interrupt sources
	SPICON = BIT0    // Enable SPI 
		   + BIT2 + BIT9  // Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
		   + BIT11;		   //Continuous transfer mode + Enable IRQ on 1 byte transfer
	IRQEN = BIT12 + BIT13; 		// Enable SPI + XIRQ0interrupt

	ucStoreCount = 0;
	while (1)
	{
		if (ucWaitingForXIRQ0 == 0)	// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ0 = 1;
			SPITX = 0xA0;
		}
	}
   return 0x00;
}

void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int SPIMSTATUS = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register

	if ((IRQSTATUS & BIT12) == BIT12)	//If SPI Slave interrupt source
	{
	   SPIMSTATUS = SPISTA;

	   if ((SPIMSTATUS & BIT5) == BIT5) // If SPI Slave Tx IRQ
	   {
			if (ucStoreCount < 6)		   // Have max 6 bytes been sent?
				SPITX = szStoreData[ucStoreCount++];	 // Load Tx buffer
			if (ucStoreCount >=6)
				ucStoreCount = 0;
	   }
	   if ((SPIMSTATUS & BIT6) == BIT6) // If SPI Slave Rx IRQ
	   {
			if (ucStoreCount < 6)		   // Have max 6 bytes been received?
			{
				szStoreData[ucStoreCount] = SPIRX;  // Read Rx buffer
				SPITX = szStoreData[ucStoreCount];
				ucStoreCount++;
				if (ucStoreCount >=6)
					ucStoreCount = 0;
			}
	   }
	   // Test for SPICON.8/SPISTA.7 - remove reading fo SPIRX register
	   if ((SPIMSTATUS & BIT7) == BIT7) // If SPI Slave Rx FIFO Overflow status IRQ
	   {
		   	ucStoreCount = 0;
	   }
	}
	if ((IRQSTATUS & BIT13) == BIT13)	//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;			// Enable Write from Slave.
	}
}


